Unit psBarcodeComp;

{
Library name  ...................: Barcode studio
Classes implemented..............: TpsBarcodeComponent ....
Version..........................:
Status...........................: Final
Author...........................: Peter CIRIP , PSOFT , Slovak republic
Comments, bugs, suggestions to...: peter@psoft.sk, peter@psoft.sk
Authors homepage.................: http://www.psoft.sk
.................................: http://barcode-software.eu
Language.........................: English
Platform.........................: Windows 95/98/NT/2000/Vista/Windows7
Requires.........................: Borland Delphi 6.0 or above
                                   or Borland C++ Builder 5
}

interface

uses
  Windows, Graphics, Classes, SysUtils, ClipBrd, Dialogs,

  {$ifdef PSOFT_PROF}   psCode2D, psCodeProf, psCodeGS1,  {$endif}
  {$ifdef PSOFT_STUDIO} psCodeStudio,                     {$endif}
  {$ifdef PSOFT_GIF} GifImg, PNGImage, {$endif}

  psCodeExceptions,
  psTypes,
  psCodeSpecs,
  psCodeFNlite,
  psCodeRes;

type
    TpsBarcodeComponent = class;

    TpsExport = class(TPersistent)
    private
      FExWidth     : Integer;  // width & Height for export operations
      FExHeight    : Integer;
      FExFileName  : TFileName; // file to export
    public
    published
      property Width : Integer read FExWidth write FExWidth;
      property Height: Integer read FExHeight write FExHeight;
      property FileName:TFileName read FExFileName write FExFileName;
    end;

    TpsParams = class(TPersistent)
    private
      FParentBarcode: TpsBarcodeComponent;
      FECI          : Integer;
      FSegmentIndex : Integer;
      FSegmentCount : Integer;
      FRatio        : TpsRatio;
      FZoomKoef: Double;
      FExport       : TpsExport;

      {$ifdef PSOFT_PROF}
          FPDF417 : TpsPDF417Params;
          FGS1    : TpsGS1;
          FFileID : String;
      {$endif}
      {$ifdef PSOFT_STUDIO}
          FdmParams     : TpsDataMatrixParams;
          FQrParams     : TpsQRParams;
          FAztecParams  : TpsAztecParams;
      {$endif}
      {$ifdef PSOFT_PROF}
          procedure SetFileID(const Value:String);
      {$endif}

      procedure UpdateBarcode;
      procedure SetECI(const Value: Integer);
      procedure SetSegmentCount(const Value: Integer);
      procedure SetSegmentIndex(const Value: Integer);
      procedure SetRatio(const Value: TpsRatio);
      procedure SetZoomKoef(const Value: Double);
    public
      constructor Create(ParentEAN:TpsBarcodeComponent);
      destructor  Destroy;  override;
      procedure   Assign(Source:TPersistent); override;
      function    BarcodeObject:TpsBarcodeComponent;
    published
//      property FNC1_Type:TpsFNC1Type read FFNC1Type write SetFNC1Type default fnc1None;
//      property FNC1_InputType:TpsFNC1InputType read fFNC1InputType write SetFNC1InputType;
      property ExportBarcode:TpsExport read FExport write FExport;
      property SegmentCount:Integer read FSegmentCount write SetSegmentCount default 1;
      property SegmentIndex:Integer read FSegmentIndex write SetSegmentIndex default 0;
      property ECI:Integer read FECI write SetECI default -1;
      property Ratio:TpsRatio read FRatio write SetRatio default ra20;
      property ZoomKoef:Double read FZoomKoef write SetZoomKoef;
      {$ifdef PSOFT_PROF}
          property GS1    : TpsGS1          Read FGS1 write FGS1;
          property PDF417 : TpsPDF417Params Read FPDF417 write FPDF417;
          property FileID : String          read FFileID write SetFileID;
      {$endif}

      {$ifdef PSOFT_STUDIO}
          property Aztec:TpsAztecParams read FAztecParams write FAztecParams;
          property DataMatrix:TpsDataMatrixParams read FdmParams write FdmParams;
          property QRCode:TpsQRParams read FqrParams write FqrParams;
      {$endif}
    end;

    TpsCaptionPosition = (cpUpper, cpHuman, cpBottom);

    TpsBarcodeCaption = class(TPersistent)
    private
       FParentBarcode: TpsBarcodeComponent;
       FVisible       : Boolean;
       FText          : TStrings;
       FFont          : TFont;
       FAutoSize      : Boolean;
       FAlignment     : TAlignment;
       FAutoCaption   : Boolean;
       FMaxHeight     : Integer;
       FParentFont    : Boolean;
       FLineSpacing   : Integer;
       FBgColor       : TColor;
       FPosition      : TpsCaptionPosition;
       procedure FontChanged(Sender: TObject);
       procedure UpdateBarcode;
    protected
       function  GetVisible:Boolean;
       procedure SetVisible(const Value:Boolean);
       function  GetText:TStrings;
       procedure SetText(const Value:TStrings);
       function  GetFont:TFont;
       procedure SetFont(const Value:TFont);
       function  GetAutoSize:Boolean;
       procedure SetAutoSize(const Value:Boolean);
       function  GetAlignment:TAlignment;
       procedure SetAlignment(const Value:TAlignment);
       function  GetAutoCaption:Boolean;
       procedure SetAutoCaption(const Value:Boolean);
       function  GetMaxHeight:Integer;
       procedure SetMaxHeight(const Value:Integer);
       function  GetParentFont:Boolean;
       procedure SetParentFont(const Value:Boolean);
       function  GetLineSpacing:Integer;
       procedure SetLineSpacing(const Value:Integer);
       function  GetBgColor:TColor;
       procedure SetBgColor(const Value:TColor);

       procedure   Paint(C:TCanvas; var R:TRect; RC:TPoint; wi:Integer); virtual;
    public
       constructor CreateEAN(AOwner:TpsBarcodeComponent; Pos:TpsCaptionPosition);
       destructor  Destroy; override;
       procedure   Assign(Source: TPersistent); override;
       procedure   UpdateCaption;
    published
       property Visible     : Boolean     Read GetVisible     Write SetVisible;
       property Text        : TStrings    Read GetText        Write SetText;
       property Font        : TFont       Read GetFont        Write SetFont;
       property AutoSize    : Boolean     Read GetAutoSize    Write SetAutoSize;
       property Alignment   : TAlignment  Read GetAlignment   Write SetAlignment;
       property AutoCaption : Boolean     Read GetAutoCaption Write SetAutoCaption;
       property MaxHeight   : Integer     Read GetMaxHeight   Write SetMaxHeight;
       property ParentFont  : Boolean     Read GetParentFont  Write SetParentFont;
       property LineSpacing : Integer     Read GetLineSpacing Write SetLineSpacing;
       property BgColor     : TColor      Read GetBgColor     Write SetBgColor;
    end;


    TpsQuietZoneIndicatorStyle = (qzNone, qzLines, qzTriangel, qzRectangle);

    TpsQuietZone = class(TPersistent)
    private
        FIndicatorSize  : Integer;
        FStyle          : TpsQuietZoneIndicatorStyle;
        FPen            : TPen;
        FSpace          : array[0..3] of Integer;
        FParentBarcode  : TpsBarcodeComponent;
        FVisible        : Boolean;
        procedure SetIndicatorSize(const Value: Integer);
        procedure SetStyle(const Value: TpsQuietZoneIndicatorStyle);
        procedure SetPen(const Value: TPen);
        function  GetSpace(Index: Integer): Integer;
        procedure SetSpace(Index: Integer; Value: Integer);
        procedure SetVisible(const Value: Boolean);
    protected
        procedure   Clear;
        procedure   UpdateBarcode;
    public
        constructor CreateEan(Owner: TpsBarcodeComponent);
        destructor  Destroy; override;
        procedure   Paint(C: TCanvas; var R: TRect);
        procedure   Assign(Source: TPersistent); override;
        procedure   Change(Sender: TObject);
        procedure   SetZone(ValueLeft, ValueTop, ValueWidth, ValueHeight: Integer);
        procedure   ClearZone;
    published
        property Pen: TPen read FPen write SetPen;
        property Style: TpsQuietZoneIndicatorStyle read FStyle write SetStyle default qzLines;
        property IndicatorSize: Integer read FIndicatorSize write SetIndicatorSize default 5;
        property Left: Integer index 0 read GetSpace write SetSpace default 0;
        property Top: Integer index 1 read GetSpace write SetSpace default 0;
        property Right: Integer index 2 read GetSpace write SetSpace default 0;
        property Bottom: Integer index 3 read GetSpace write SetSpace default 0;
        property Visible:Boolean read FVisible write SetVisible default False;
    end;

  TpsHorzLines = class(TPen)
  private
      FBarcodeComp: TpsBarcodeComponent;
      FLinesCount : Integer;
      procedure SetLinesCount(Value:Integer);
  public
      constructor CreateEAN(AOwner:TpsBarcodeComponent);
      procedure   Assign(Source:TPersistent); override;
  published
      property LinesCount : Integer Read FLinesCount Write SetLinesCount Default 0;
  end;


  IpsBarcodeInterface = interface
    ['{872CF3D9-9900-4A50-ABFC-0D546AE71E5E}']

    function  GetAbout:String;

    function  BarcodeComponent:TpsBarcodeComponent;

    function  GetOptions:TpsBarcodeOptions;
    procedure SetOptions(const Value:TpsBarcodeOptions);

    function  GetBackgroundColor: TColor;
    procedure SetBackgroundColor(const Value:TColor);

    function  GetAngle: Integer;
    procedure SetAngle(const Value:Integer);

    function  GetBarcode:String;
    procedure SetBarcode(const Value:String);

    function  GetBarcodeSymbology:TpsBarcodeSymbology;
    procedure SetBarcodeSymbology(const value :TpsBarcodeSymbology);

    function  GetLinesColor:TColor;
    procedure SetLinesColor(const Value:TColor);

    function  GetQuietZone:TpsQuietZone;
    procedure SetQuietZone(const Value:TpsQuietZone);

    function  GetHorzLines:TpsHorzLines;
    procedure SetHorzLines(const Value:TpsHorzLines);

    function  GetParams:TpsParams;
    procedure SetParams(const Value:TpsParams);

    function  GetCaptionUpper:TpsBarcodeCaption;
    procedure SetCaptionUpper(const Value:TpsBarcodeCaption);

    function  GetCaptionBottom:TpsBarcodeCaption;
    procedure SetCaptionBottom(const Value:TpsBarcodeCaption);

    function  GetCaptionHuman:TpsBarcodeCaption;
    procedure SetCaptionHuman(const Value:TpsBarcodeCaption);

    function  GetErrorInfo:TpsBarcodeException;
    // ----------------------------------------------------------------------
    // psBarcode properties
    // ----------------------------------------------------------------------

    property  CaptionHuman:TpsBarcodeCaption read GetCaptionHuman write SetCaptionHuman;
    property  CaptionBottom:TpsBarcodeCaption read GetCaptionBottom write SetCaptionBottom;
    property  CaptionUpper:TpsBarcodeCaption read GetCaptionUpper write SetCaptionUpper;
    property  Params:TpsParams read GetParams write SetParams;
    property  HorzLines:TpsHorzLines read GetHorzLines write SetHorzLines;
    property  QuietZone:TpsQuietZone read GetQuietZone write SetQuietZone;
    property  LinesColor:TColor read GetLinesColor write SetLinesColor;
    property  BarcodeSymbology:TpsBarcodeSymbology read GetBarcodeSymbology write SetBarcodeSymbology;
    property  Barcode:String read GetBarcode write SetBarcode;
    property  Options:TpsBarcodeOptions read GetOptions write SetOptions;
    property  BackgroundColor:TColor read GetBackgroundColor write SetBackgroundColor;
    property  Angle:Integer read GetAngle write SetAngle;
    property  About:String  read GetAbout;
    property  ErrorInfo:TpsBarcodeException read GetErrorInfo;
  end;

  // TOnUpdateBarcodeParent = procedure (Sender:TObject) of object;

  TpsBarcodeComponent = class(TComponent, IpsBarcodeInterface)
  private
         FBarCode        : string;
         FBarcodeSymbology     : TpsBarcodeSymbology;
         FLinesColor     : TColor;
         FFontAutoSize   : Boolean;
         FBackgroundColor: TColor;
         FAngle          : Integer;
         FOnChangeTypCode: TNotifyEvent;
         FOnChangeBarCode: TNotifyEvent;
         FCaptionUpper   : TpsBarcodeCaption;
         FCaptionBottom  : TpsBarcodeCaption;
         FCaptionHuman   : TpsBarcodeCaption;

         FBars           : TStringList;
         FHorzLines      : TpsHorzLines;

         FQuietZone      : TpsQuietZone;

         FParams         : TpsParams;
         FOptions        : TpsBarcodeOptions;

         {$ifdef PSOFT_STUDIO}
              FPixels         : TpsMatrix;
         {$endif}
         FErrorInfo      : TpsBarcodeException;
         FOnUpdate       : TNotifyEvent;

         FOnUpdateBarcodeParent: TNotifyEvent;
    FCharWidth: Double;

         function    GetAbout:String;
         function    GetBackgroundColor: TColor;
         procedure   SetBackgroundColor(const Value:TColor);

         function    GetBarCode:String;
         procedure   SetBarCode(const Value:String);

         function    GetBarCodeSymbology:TpsBarCodeSymbology;
         procedure   SetBarCodeSymbology(const Value:TpsBarCodeSymbology);

         function    GetOptions:TpsBarcodeOptions;
         procedure   SetOptions(const Value: TpsBarcodeOptions);

         function    GetAngle: Integer;
         procedure   SetAngle(const Value:Integer);

         function    GetLinesColor:TColor;
         procedure   SetLinesColor(const Value:TColor);

         function    GetQuietZone:TpsQuietZone;
         procedure   SetQuietZone(const Value: TpsQuietZone);

         function    GetHorzLines: TpsHorzLines;
         procedure   SetHorzLines(const Value: TpsHorzLines);

         function    GetParams: TpsParams;
         procedure   SetParams(const Value: TpsParams);

         function    GetCaptionUpper:TpsBarcodeCaption;
         procedure   SetCaptionUpper(const Value:TpsBarcodeCaption);

         function    GetCaptionBottom:TpsBarcodeCaption;
         procedure   SetCaptionBottom(const Value:TpsBarcodeCaption);

         function    GetCaptionHuman:TpsBarcodeCaption;
         procedure   SetCaptionHuman(const Value:TpsBarcodeCaption);

         function    GetErrorInfo:TpsBarcodeException;

         procedure   UpdateCaptions;


         function    Codabar(ck:string):String;
         function    AbcCodabar(const ck:string):string;
         function    Code39Standard(const ck:string):string;
         function    Code39Full(const ck:string):string;
         function    Code93Standard(ck:string):string;
         function    Code93Full(ck:string):string;
         function    Code128(const ck:string):string;
         function    Code128A(const ck:string):string;
         function    Code128B(const ck:string):string;
         function    Code128C(ck:string):string;
         function    Code128Finalize(enc:string):string;
         function    UccEan128(const ck:string):string;
         function    EanAddOn(const ck:String; le:Integer):String;
         function    Ean8(const ck:string):string;
         function    Ean13(const ck:string):string;
         function    UPCA(const ck:String):String;
         function    UPCE(const ck:String):String;
         // function    ITF14(ck:string):string;
         function    Plessey(ck:String):String;
         function    Datalogic(const ck:string):String;
         function    Matrix(const ck:string):String;
         function    Industrial(ck:string):String;
         function    Interleaved(ck:string):String;
         function    IATA(ck:string):String;
         function    Invert(ck:string):string;
         function    Coop(ck:string):String;
         function    Code11(ck:string):String;
         function    PostNet(ck:string):string;
         function    PZN(ck:String):String;

         function    CheckEAN8 (var s:String):Boolean;
         function    CheckEAN13(var s:String):Boolean;
         function    CheckUPCA (var s:String):Boolean;
         function    TestCharsInSet(s:String):Boolean;
         procedure   HorzLinesChange(Sender:TObject);

         function    RoyalMail(ck: string): string;
         function    SingaporeCode(ck: string): string;
         function    PlanetCode(ck: string): string;
         function    Monarch(ck: String): String;
         function    PostBar(ck: string): string;
         function    SwissPostal(ck: string): string;
         function    Dutch4StatePostal(ck: string): string;
         function    State4(ck: string): string;
         function    Fim(ck: string): string;

         procedure   UpdateBarcode;
         procedure   DoUpdate;
         procedure   PaintBarCode2(C: TCanvas; R: TRect);
         function    GetLastError: TpsBarcodeException;
         procedure   SetlastError(Value: TpsBarcodeException);
         procedure SetCharWidth(const Value: Double);
  protected
         function    GetBarCodeLines(BC:String):String;
  public
         constructor Create(AOwner:TComponent); override;
         constructor CreateFromControl(AOwner:TComponent;
              UpdateProc:TNotifyEvent); // TOnUpdateBarcodeParent);
         constructor CreateSymbology(AOwner:TComponent;
              NewSymbology:TpsBarcodeSymbology; Value:String);
         destructor  Destroy;                   override;

         procedure   ChangeSymbology(dir: TpsDirection; OnlySupported: Boolean);

         function    BarcodeHint:String;

         function    SymbologyName:String;

         function    Bars:TStringList;
         function    Zoom(ABarCode:string; AWidth:integer; ASecurity:Boolean):string;

         function    Compress128(s: String): string;
         function    SolveGS1:String;

         procedure   SetExParams(w,h:Integer);
         procedure   BarcodeRaise; overload;
         procedure   BarcodeRaiseStr(err:TpsBarcodeExceptionCode;
              str:String=''; hlp:Integer=0);
         procedure   BarcodeRaise(err:TpsBarcodeExceptionCode; idx:Integer=0;
              str:String=''; hlp:Integer=0); overload;
         procedure   BarcodeRaise(err:TpsBarcodeExceptionCode; P1,P2:Integer ); overload;
         procedure   BarcodeRaise(err:TpsErrRecord); overload;

         {$ifdef PSOFT_STUDIO}
           function    Pixels:TpsMatrix;
         {$endif}
         procedure   BarcodeComplete;
         procedure   PaintBarCode(C: TCanvas; R: TRect);
         function    GetHint: String;

         function    MinWidth  : Integer;
         function    MinHeight(DefHeightPercent:Integer=40) : Integer;
         function    GetSetOfChars:string;
         function    GetSetOfCharsVisible:String;
         function    CheckBarCode(S:String):Boolean;

         function    BI:TpsBarCodeInfo;
         function    ExRect:TRect;
         procedure   SetSize(w,h:Integer);

         function    GetBarcodeInfo:TpsBarCodeInfo;
         procedure   Assign(Source:TPersistent); override;
         procedure   AssignOnlyBarcode(Source:TpsBarcodeComponent); virtual;
         function    BarcodeTypeName:String;

         procedure   PaintBarCodeMM(C: TCanvas; left,top,width,height:double);
         function    BarcodeComponent:TpsBarcodeComponent;

         // exports procedures
         function   ExCheck:Boolean;  // check export parameters
         function   CopyToClipboard:Boolean;
         function   CopyToClipboardWMF:Boolean;
         function   SaveToFile:Boolean;
         function   SaveAsBmp:Boolean;
         function   SaveAsWmf:Boolean;
         {$ifdef PSOFT_JPEG}
            function SaveAsJpeg:Boolean;
         {$endif}
         {$ifdef PSOFT_GIF}
            function SaveAsGIF:Boolean;
            function SaveAsPNG:Boolean;
         {$endif}
  published
         property About             : String         Read GetAbout Stored False;
         property BackgroundColor   : TColor         Read GetBackgroundColor Write SetBackgroundColor;
         property Options           : TpsBarcodeOptions read FOptions write SetOptions;
         property BarCode           : string         Read GetBarCode         Write SetBarCode;
         property BarcodeSymbology  : TpsBarcodeSymbology    Read GetBarcodeSymbology Write SetBarcodeSymbology;
         property QuietZone         : TpsQuietZone read GetQuietZone write SetQuietZone;
         property HorzLines         : TpsHorzLines Read GetHorzLines      Write SetHorzLines;
         property ErrorInfo         : TpsBarcodeException read GetlastError write SetLastError;
         property LinesColor        : TColor         Read GetLinesColor     Write SetLinesColor;

         // property LabelMask         : string         Read FLabelMask       Write SetLabelMask;
         property Angle             : Integer        Read GetAngle         Write SetAngle             Default 0;
         property CaptionUpper      : TpsBarcodeCaption Read GetCaptionUpper   Write SetCaptionUpper  Stored True;
         property CaptionBottom     : TpsBarcodeCaption Read GetCaptionBottom  Write SetCaptionBottom Stored True;
         property CaptionHuman      : TpsBarcodeCaption Read GetCaptionHuman   Write SetCaptionHuman  Stored True;

         property Params:TpsParams read GetParams write SetParams;

         property OnChangeTypCode:TNotifyEvent
              Read FOnChangeTypCode Write FOnChangeTypCode;
         property OnChangeBarCode:TNotifyEvent
              Read FOnChangeBarCode Write FOnChangeBarCode;
         property OnUpdate:TNotifyEvent
              Read FOnUpdate write FOnUpdate;
         property CharWidth:Double read FCharWidth write SetCharWidth;
  end;

  TpsBarcodeApplication=class(TComponent)
  private
      FCaption: string;
      FOptions: TpsBarcodeEditorOptions;
      function GetAbout: string;
  published
      constructor Create(AOwner:TComponent); override;
  public

  published
      property About:string read GetAbout stored False;
      property Caption:string read FCaption write FCaption;
      property Options:TpsBarcodeEditorOptions read FOptions write FOptions;
  end;



  procedure psUpdateBarcode(bc:TComponent);


  //----------------------------------------------------------------
  // procedures to paint barcode on the TCanvas, window or device
  //----------------------------------------------------------------

  // paint barcode on the TCanvas
  procedure PaintBarCode(C:TCanvas; R:TRect; E:TpsBarcodeComponent);

  // paint barcode using device context
  procedure PaintBarCodeDC(dc:HDC; R:TRect; E:TpsBarcodeComponent);

  // paint barcode using window handle
  procedure PaintBarCodeHandle(HWnd:THandle; R:TRect; E:TpsBarcodeComponent);

  function  IAmDemo:Boolean;

  function  IsBookland(ISBN:String):Boolean;
  function  BooklandToEAN(s:String; T:TpsBarcodeSymbology):String;
  function  CalcEan8CheckDigit(s:String):Char;
  function  CalcEan13CheckDigit(s:String):Char;
  function  CalcISBNCheckDigit(s:String):Char;

  procedure DrawDemo(C:TCanvas; R:TRect; rc:TPoint; A:Integer; bg:TColor);

var ofs : TpsOffsets;

implementation

