unit psCodeProf;

interface

{$I psBarcode.inc}

uses Classes, Windows, Graphics, SysUtils, DateUtils,
  psCodeRes, psTypes, psCodeFNLite, psCodeExceptions;

type
        longword4 = array[0..3] of longword;

        TInteger5 = array[0..4] of Integer;
        TByte6    = array[0..5] of Byte;

// ****************************************************************************
// *** procedured/functions for work with PDF417
// ****************************************************************************

    TpsPDF417Mode = (psPDF417Alphanumeric, psPDF417Binary, psPDF417BinaryHex, psPDF417Numeric, psPDF417AutoCode);

    TpsPDF417ErrorCorrection = (psPDF417AutoEC,
        psPDF417Error0, psPDF417Error1, psPDF417Error2,
        psPDF417Error3, psPDF417Error4, psPDF417Error5,
        psPDF417Error6, psPDF417Error7, psPDF417Error8 );
    TpsPDF417Kind = (pkStandard, pkTruncated, pkMicro);

    TpsPDF417Params=class(TPersistent)
    private
      FBarcode        : TComponent;
      FMode           : TpsPDF417Mode;
      FRows           : Integer;
      FCols           : Integer;
      FSecurityLevel  : TpsPDF417ErrorCorrection;
      FFileSize       : Integer;
      FSender         : String;
      FAddresse       : string;
      FTimeStamp      : TDateTime;
      FKind           : TpsPDF417Kind;
      FUseMacro       : Boolean;
      FFileName       : String;
      FCheckSum       : Integer;
      FUsedRows       : Integer;
      FUsedCols       : Integer;
      FUsedECL        : TpsPDF417ErrorCorrection;
      procedure       UpdateBarcode;
    protected
      procedure SetMode(Value:TpsPDF417Mode);
      procedure SetSecurityLevel(Value:TpsPDF417ErrorCorrection);
      procedure SetRows(Value:Integer);
      procedure SetCols(Value:Integer);
      procedure SetAddresse(const Value: string);
      procedure SetFileSize(const Value: Integer);
      procedure SetSender(const Value: String);
      procedure SetTimeStamp(const Value: TDateTime);
      procedure SetKind(Value:TpsPDF417Kind);
      procedure SetFileName(const Value: String);
      procedure SetChecksum(const Value: Integer);
      procedure SetUseMacro(Value: Boolean);
    public
      constructor CreateBarcode(AOwner:TComponent);
      procedure   Assign(Source: TPersistent); override;

      function    BarcodeComponent:TComponent;
      //function  OptionalFieldCount:Integer;
    published
      property  Mode:TpsPDF417Mode Read FMode Write SetMode;
      property  Rows:Integer Read FRows Write SetRows Default 0;
      property  Cols:Integer Read FCols Write SetCols Default 10;
      property  SecurityLevel: TpsPDF417ErrorCorrection Read FSecurityLevel Write SetSecurityLevel;
      property  TimeStamp:TDateTime read FTimeStamp write SetTimeStamp;
      property  Sender:String read FSender write SetSender;
      property  Addresse:string read FAddresse write SetAddresse;
      property  FileSize:Integer read FFileSize write SetFileSize;
      property  Kind:TpsPDF417Kind read FKind write SetKind;
      property  FileName:String read FFileName write SetFileName;
      property  Checksum:Integer read FChecksum write SetChecksum;
      property  UseMacro:Boolean read FUseMacro write SetUseMacro;
      property  UsedCols:Integer read FUsedCols stored False;
      property  UsedRows:Integer read FUsedRows stored False;
      property  UsedECL :TpsPDF417ErrorCorrection read FUsedECL stored False;
    end;

    procedure psPDF417GetLines(bc:TComponent; L:TStringList);

//    procedure psPDF417GetPixels(S:String; var Cols, Rows:Integer;
//          BarcodeObject:TComponent);

// ****************************************************************************
// *** HugeInt functions - 128 bit arithmetic - need in IntelligentMailbarcode
// *** used in PDF417 for increase spped ...
// ****************************************************************************
type
    Hugeint = packed array [0..3] of longword;

    function LongWordToHuge(const x: Longword): Hugeint; overload;
    function Int64ToHuge(const x: Int64): Hugeint; overload;
    function LongIntToHuge(const x: Integer): Hugeint; overload;
    function HugeAdd(x: Hugeint; y: Hugeint): Hugeint;
    function MultiplyWithCarry(x: Longword; y: Longword; var Carry: Longword): LongWord;
    function HugeMulInt(x: Hugeint; y: Longword): Hugeint;
    function HugeGetBit(h:HugeInt; bit:Integer):Integer;
    function DivideWithRemainder(x: Longword; y: Longword; var Remainder: Longword): Longword;
    function HugeDivInt(x: Hugeint; y: Longword): Hugeint;
    function HugeModInt(dividend: Hugeint; divisor: Longint): Longint;

// **************************************************************************
// *** Telepen barcode
// **************************************************************************
    function Telepen(ck:String):string;
    function TelepenNumeric(ck:String):string;

// **************************************************************************
// *** IntelligentMail
// **************************************************************************
    function IntelligentMailGetLines(ToEncode:String):String;
    function IntelligentMailCheck(ToEncode:String; BC:TObject):Boolean;

// **************************************************************************
// *** Australia Post barcode
// **************************************************************************
    function AustraliaPostGetLines(var ToEncode:String; ShowChecksum:Boolean):String;
    function AustraliaPostCheck(ToEncode:String; BC:TObject):Boolean;

// **************************************************************************
// *** Postbar , CPC 4-state
// **************************************************************************
    function  CPC4State(ck:string):string;
    function  CPC4StateCheck(var ck:string):Boolean;

// **************************************************************************
// **** Used for encoding Code16K                                         ***
// **************************************************************************



// **************************************************************************
// **** bcChromoCode - color barcode symbology                            ***
// **************************************************************************
    function  EncodeChromocode(E:TComponent):Boolean;


//    function pdf417MicroComplete(L:TStrings; cols,rows:Integer):Boolean;


procedure PaintStacked(C:TCanvas; R:TRect; bc:TComponent);


implementation

