unit psCodeStudio;

interface

uses  Windows, Classes, Math, SysUtils, Graphics,
      psTypes, psCodeFNLite, psCode2D, psCodeExceptions, psCodeGS1;

{$I psBarcode.inc}

const constDataMatrixDefaultECI = -1;

type


  TpsDataMatrixSpec = record
    No        : integer;        // Number of data regions
    DW        : integer;        // total data codewords
    EW        : integer;        // total error correction codewords
    Inter     : integer;        // Interleaved blocks
    Size      : TpsSize2D;      // Sizes in pixels
    Capacity  : TpsCapacity2D;  // Data capacity
  end;

  TdmMatrixParams = record
    CellX, CellY          : Integer; // number of cells in X and Y direction
    CellSizeX, CellSizeY  : Integer; // cell size in pixels (include control pixels)
    TotalMatrixW,
    TotalMatrixH          : Integer; // matrix size (width control pixels)
    TotalDW               : Integer; // max. datawords
    TotalEW               : Integer; // count of error correction words
    Capacity              : TpsCapacity2D;
    Inter                 : Integer;
  end;

  TpsDataMatrixEncoding = ( dmeAutomatic, dmeASCII, dmeBinary, dmeEDIFACT,
      dmeText, dmeX12, dmeC40);

  TpsDataMatrixVersion = (
      psDMAutomatic,
      psDM10x10,  psDM12x12, psDM14x14, psDM16x16,   psDM18x18,   psDM20x20,
      psDM22x22,  psDM24x24, psDM26x26, psDM32x32,   psDM36x36,   psDM40x40,
      psDM44x44,  psDM48x48, psDM52x52, psDM64x64,   psDM72x72,   psDM80x80,
      psDM88x88,  psDM96x96, ps104x104, psDM120x120, psDM132x132, psDM144x144,
      psDM8x18,   psDM8x32,  psDM12x26, psDM12x36,   psDM16x36,   psDm16x48
  );

  TpsDataMatrixParams=class(TPersistent)
  private
      FBarcode : TComponent;
      FEncoding: TpsDataMatrixEncoding;
      FVersion : TpsDataMatrixVersion;
      procedure UpdateBarcode;
      procedure SetEncoding(const Value: TpsDataMatrixEncoding);
      procedure SetVersion(const Value: TpsDataMatrixVersion);
  public
      constructor CreateBarcode(AOwner:TComponent);
      procedure   Assign(Source:TPersistent); override;
  published
      property  Encoding : TpsDataMatrixEncoding read FEncoding write SetEncoding;
      property  Version  : TpsDataMatrixVersion read FVersion write SetVersion;
  end;

  TpsQrEccLevel= (QrEccLevelL, QrEccLevelM, QrEccLevelQ, QrEccLevelH);
  TpsQrMode    = (QrAutomatic, QrNumeric, QrAlphanumeric, QrBytes, QrKanji);


    TpsQRParams = class(TPersistent)
    private
      FBarcode : TComponent;
      FMode: TpsQrMode;
      FMicroQR: Boolean;
      FEccLevel: TpsQrEccLevel;
      FVersion: Integer;
      FMask: Integer;
      FFileName     : String;
      FCheckSum     : Integer;
      {$ifdef PSOFT_SETUP}
          FDrawData: Boolean;
          FDrawMask: Boolean;
      {$endif}
      procedure UpdateBarcode;
      procedure SetEccLevel(const Value: TpsQrEccLevel);
      procedure SetMicroQR(const Value: Boolean);
      procedure SetMode(const Value: TpsQrMode);
      procedure SetVersion(const Value: Integer);
      procedure SetMask(const Value: Integer);
      procedure SetFileName(const Value: String);
      procedure SetChecksum(const Value: Integer);
    protected
    public
      FUsedVersion  : Integer;
      FUsedMask     : Integer;
      FUsedMicroQR  : Boolean;
      FUsedMode     : TpsQRMode;
      FUsedEccLevel : TpsQrEccLevel;

      constructor CreateBarcode(ParentEAN:TComponent);
      procedure   Assign(Source:TPersistent); override;
    published
      property EccLevel:TpsQrEccLevel read FEccLevel write SetEccLevel;
      property Mode:TpsQrMode read FMode write SetMode;
      property MicroQR:Boolean read FMicroQR write SetMicroQR;
      property Version:Integer read FVersion write SetVersion;
      property Mask:Integer read FMask write SetMask;
      property FileName:String read FFileName write SetFileName;
      property Checksum:Integer read FChecksum write SetChecksum;
      {$ifdef PSOFT_SETUP}
          property DrawData:Boolean read FDrawData write FDrawData stored False;
          property DrawMask:Boolean read FDrawMask write FDrawMask Stored False;
      {$endif}
      property UsedVersion:Integer read FUsedVersion stored False;
      property UsedMask:Integer read FUsedMask stored False;
      property UsedMicroQR:Boolean read FUsedMicroQR stored False;
      property UsedMode:TpsQRMode read FUsedMode stored False;
      property UsedECCLevel:TpsQrEccLevel read FUsedEccLevel stored False;
    end;

    TpsAztecType = (azStandard, azCompact, azRunes);
    TpsAztecVersion  = integer;

    TpsAztecParams = class(TPersistent)
    private
      FBarcode   : TComponent;
      FSymbolType: TpsAztecType;
      FVersion   : Integer;
      FErrorLevel: Integer;
      procedure UpdateBarcode;
      procedure SetSymbolType(const Value: TpsAztecType);
      procedure SetVersion(const Value: TpsAztecVersion);
      procedure SetErrorLevel(const Value: Integer);
    public
      constructor CreateBarcode(ParentEAN:TComponent);
      procedure   Assign(Source:TPersistent); override;
    published
      property SymbolType: TpsAztecType read FSymbolType write SetSymbolType default azStandard;
      property Version : Integer read FVersion write SetVersion;
      property ErrorLevel:Integer read FErrorLevel write SetErrorLevel default 23;
    end;


  function  dmGetMatrixParams(version:TpsDataMatrixVersion;
      TotalEncoded:integer):TdmMatrixParams;

  // toto potom prehodit do implementation sekcie
  // function  dmMapToFinal(dmParams:TdmMatrixParams; x,y:Integer):TPoint;

  //  function  psPaintDataMatrix(Value:String; bc:IpsBarcodeInterface; C:TCanvas; R:TRect):Integer;
  function  psDataMatrixGetPixels(BarcodeObject:TComponent):Integer;


  // --------------------------------------------------------------------------
  // main function for QR symbology/Micro QR ...
  // fills FBars variable of TpsCustomBarcode, for internal use only
  function  psQRCodeGetPixels(Barcode:TComponent):Integer;

  // convert ECC level to human readable text
  function  qrECCDescription(ecc:TpsQrEccLevel):string;
  function  qrModeDescription(ecc:TpsQrMode):string;

  function  psAztecGetPixels(Barcode:TComponent):Integer;


